# CellTracking.py
# By MW, Jun 2013
# GPLv3+
#
# This is the core of the program. Please invoke this script when creating a new project. It provides any backend function and can encapsulate many graphical primitives.

VERSION = "1.0"
RELEASE = "a4"

def get_version() :
    """Function that returns the version of the CellTracking program. Mostly used for automatic generation of binary exports.
    :rtype: (version, release)"""
    return (VERSION, VERSION+RELEASE)

# ==== Importing modules
import imp, logging, sys


logging.basicConfig(filename='./log/main.log', format='%(asctime)s:%(levelname)s : %(message)s', level=logging.DEBUG)  # Starting logging system
logging.info("Starting CellTracking, v%s", VERSION+RELEASE)    # Starting logging module

logging.info("Loading modules...")
        
logging.debug("Loading Movies module...")
Movies = imp.load_source("Movies", "./bin/Movies/Movies.py") # Loading the Movies module that manages interaction with sequences
logging.debug("...done")

logging.debug("Loading States module...")
States = imp.load_source("States", "./bin/States/States.py") # Loading the States module
logging.debug("...done")


logging.debug("Loading Players module...")
Players = imp.load_source("Players", "./bin/Players/Players.py") # Loading the Players module
Players = Players.Players()
logging.debug("...done")


logging.info("...all modules loaded!")

if __name__ == "__main__" :
    if len(sys.argv)==1 :
        pass
    elif sys.argv[1] == '-v' :
        print VERSION+RELEASE

logging.info("Stopping CellTracking, v%s", VERSION+RELEASE)    # Stopping program
logging.info("------------------------------------------")
