# CellTracking.py
# By MW, Jun 2013
# GPLv3+
#
# Class managing a cell
import logging, cv2, imp, numpy

TimeTrack = imp.load_source("TimeTrack", './bin/Movies/Cells/TimeTrack.py')

class Cell :
    def __init__(self, movie, name, begin, end) :
        self.movie = movie
        self.name = name
        self.begin = begin
        self.end = end
        # Initialize timetracks (empty)
        self.cell_pol = TimeTrack.TimeTrack() # The cell inner polygon
        self.cell_ref = TimeTrack.TimeTrack() # The cell reference polygon
        self.center_pol = 'mean'
        self.center_ref = None
        self.handle_dist = 8 # The tolerance distance of the cell
    
    def reset_polygon(self, pol, t) :
        self.cell_pol = TimeTrack.TimeTrack()
        self.cell_pol.add_timepoint(t, pol)
        self.cell_ref = TimeTrack.TimeTrack()
        self.generate_ref()
    def set_name(self, n) :
        self.name = str(n)
    def get_name(self) :
        return self.name
    def generate_ref(self) :
        pass
    def is_point_in_cell(self, pt, t) :
        pol = self.cell_pol.get_polygon(t)
        if cv2.pointPolygonTest(numpy.array([pol], dtype=numpy.int32), pt, False) == 1.0 :
            return True
        else :
            return False
    def is_point_in_ref(self, pt, t) :
        logging.error("Is point in ref not implemented yet, returning False")
        #pol = self.cell_ref.get_polygon(t)
        #if cv2.pointPolygonTest(numpy.array([pol], dtype=numpy.int32), pt, False)==1.0 :
        #    return True
        #else :
        #    return False
        return False
    def is_point_on_cell_handle(self, pt, t) :
        """Tests if the point lies within a given distance of a handle (an edge, and maybe a center"""
        pol = self.cell_pol.get_polygon(t)
        return self.point_on_handle(pol, pt, self.center_pol)        
    def is_point_on_ref_handle(self, pt, t) :
        pol = self.cell_ref.get_polygon(t)
        return self.point_on_handle(pol, pt, self.center_ref)
    def point_on_handle(self, pol, tup, center_type) :
        """Internal function returning the selected edge of the polygon (if 
        any), the center or None"""
        selected = None
        for (i,p) in enumerate(pol) :
            if self.dist(p, tup, self.handle_dist) :
                selected = i
        if center_type == 'mean' :
            center = numpy.array(pol).mean(axis=0)
            if self.dist(tup, center, self.handle_dist) :
                selected = 'center'
        return selected
    
    def dist(self, pt1, pt2, rad) :
        return ((pt1[0]-pt2[0])**2+(pt1[1]-pt2[1])**2 < rad**2)
